#!/bin/sh
#=======================================================================
# This script sends its argument PostScript file(s), or GIF, JPEG, PNG,
# Sun raster, and TIFF image files, to a duplex PostScript printer, such
# as the HP LaserJet 4si or 5si, or the Lexmark Optra S 2450.
#
# The script suppresses (for print-ps-oneside) or selects (for
# print-ps-twoside) duplex printing, but starts the first output page on
# the second sheet of paper.  The first sheet of paper will contain the
# normal job header.  This is useful for printing typeset output that
# has different even and odd side margins.
#
# Alternatively, for some printers, including the Lexmark Optra SC
# 1275 color printer, it sends the output for printing on
# transparencies.  In such a case, the script suppresses duplex
# printing, since that would be completely useless on transparent
# output!
#
# The script behavior is controlled by its name, so one script serves
# two purposes.
#
# Usage:
#	print-ps-oneside      [-P printer] [-r resolution-in-dpi] file1 file2 ...
#	print-ps-transparency [-P printer] [-r resolution-in-dpi] file1 file2 ...
#	print-ps-twoside      [-P printer] [-r resolution-in-dpi] file1 file2 ...
#
# If no files are specified, then the standard input is read and printed.
#
# The -R option can be used to set the printer's default resolution,
# usually 300dpi, 600dpi, or 1200dpi.  With some printers, printing
# speed doubles when resolution is reduced (from 1200dpi to 300dpi on
# the Lexmark Optra S 2450).
#
# PostScript, DVI, Tektronix, and X79 file formats are recognized, and
# converted to PostScript if necessary.  All other file formats must be
# manually converted to PostScript first.
#
# [11-Jan-2001] -- Adapt from print-ps-oneside.
#
# [30-Nov-2000] -- Add jwb129 to list of duplex printers.
#
# [01-Dec-1997] -- Add lex3fl to list of duplex printers.
#
# [03-Oct-1997] -- Add -#copies option support.
#
# [27-Sep-1997] -- Add -r option support.
#
# 		   Add automatic filtering of GIF, JPEG, PNG, Sun
# 		   raster, and TIFF image files.
#
#		   Replace most if-then-else-fi statements by case-esac
#		   to clarify coding.
#
# 		   Combine print-ps-oneside and print-ps-twoside into a
# 		   single script.
#
# [20-Sep-1997] -- add Lexmark printer support
#
# [05-Mar-1996] -- original version
#=======================================================================

PBMPLUS=/usr/local/pbmplus
PLT=/usr/local/plot79
TMPDIR=${TMPDIR-/tmp}

AWK=/usr/local/bin/gawk		# need gawk, rather than nawk or awk, to handle binary files
DJPEG=/usr/local/bin/djpeg
DVIALW=/usr/local/bin/dvialw
GIFTOPNM=$PBMPLUS/giftopnm
PNGTOPNM=$PBMPLUS/pngtopnm
PNMTOPS=$PBMPLUS/pnmtops
RASTTOPNM=$PBMPLUS/rasttopnm
TIFFTOPNM=$PBMPLUS/tifftopnm
TEKALW=$PLT/tekalw
X79PS=$PLT/x79ps

PNMTOPSFLAGS=-noturn		# suppress auto-rotation of landscape orientation image files

exitcode=0
COPIES=1
FILES=
RESOLUTION=

while test $# -gt 0		# process command-line arguments
do
    case $1 in
    -#)
	    COPIES=$2
	    shift
	    ;;
    -#[0-9]*)
	    COPIES=`echo $1 | $AWK '{print substr($0,3)}'`
	    ;;
    -P[a-z]*)			# "-Pprintername"
	    PRINTER=`echo $1 | $AWK '{print substr($0,3)}'`
	    ;;
    -P)				# "-P printername"
	    PRINTER=$2
	    shift
	    ;;
    -[Rr][0-9]*)
	    RESOLUTION=`echo $1 | $AWK '{print substr($0,3)}'`
	    ;;
    -[Rr])
	    RESOLUTION=$2
	    shift
	    ;;
    -*)
	    echo "WARNING: Unrecognized option ["$1"] ignored"
	    exitcode=1
	    ;;
    *)	    FILES="$FILES $1"
	    ;;
    esac
    shift
done

# Check for printers that support duplex printing:
case $PRINTER in
b129lab*|jwb120|lex120|lex3fl|inscc300|jwb128|jwb229-2|jwb320-2)
    ;;
*)
    case $0 in		# [11-Jan-2001] NEW!
    *twoside)
	echo "WARNING: $PRINTER does not support duplex printing: printing simplex instead"
	exitcode=1
	;;
    *)
	;;
    esac
    ;;
esac

export PRINTER
INFILE=$TMPDIR/print.$$.in
OUTFILE=$TMPDIR/print.$$.out
LPRFILE=$TMPDIR/print.$$.tmp

# Clean up all temporary files on termination
trap "/bin/rm -f $INFILE $LPRFILE $OUTFILE" 0 1 2 15

if test -z "$FILES"
then	# copy stdin to a temporary file
    cat >>$INFILE
    FILES=$INFILE
fi

for file in $FILES
do
    userfile=$file

    case `$AWK '{print substr($0,1,50); exit 0}' $file` in
    # NB: Case strings CANNOT use octal sequences: use actual characters instead

    ""*)			# DVI
	    $DVIALW -resolution:${RESOLUTION-600} -outfile:$OUTFILE $file
	    file=$OUTFILE
	    ;;
    ""*)			# Tektronix
	    $TEKALW $file $OUTFILE 2>/dev/null
	    file=$OUTFILE
	    ;;
    "#!/usr/local/plot79/x79"*)	# X11
	    $X79PS < $file >$OUTFILE 2>&1
	    file=$OUTFILE
	    ;;
    GIF87a*|GIF89a*)		# GIF
	    $GIFTOPNM < $file | $PNMTOPS $PNMTOPSFLAGS >$OUTFILE
	    file=$OUTFILE
	    ;;
    ""*)	# JPEG
	    $DJPEG -pnm < $file | $PNMTOPS $PNMTOPSFLAGS >$OUTFILE
	    file=$OUTFILE
	    ;;
    "PNG
"*)		# PNG
	    $PNGTOPNM < $file | $PNMTOPS $PNMTOPSFLAGS >$OUTFILE
	    file=$OUTFILE
	    ;;
    "Y"*)			# Sun rasterfile
	    $RASTTOPNM < $file | $PNMTOPS $PNMTOPSFLAGS >$OUTFILE
	    file=$OUTFILE
	    ;;
    II*|MM*)			# TIFF
	    $TIFFTOPNM < $file | $PNMTOPS $PNMTOPSFLAGS -dpi 1200 >$OUTFILE
	    file=$OUTFILE
	    ;;
    esac

    case `$AWK '{print substr($0,1,2); exit 0}' $file` in
    "%!")
	echo "%!PS-Adobe" >$LPRFILE

	TRAYSELECTION=false		# [11-Jan-2001] NEW!

	case $0 in			# select script behavior according to program name
	*oneside)			# simplex printing
	    DUPLEX=false
	    ;;
	*transparency)
	    DUPLEX=false
	    TRAYSELECTION=true		# [11-Jan-2001] NEW!
	    case $PRINTER in
	    inscc300)			# use top letterhead tray (1) for transparencies
		TRAYNUMBER=0		# 0 selects tray 1, sigh...
		;;
	    inscc300color)		# use bottom tray (2) for transparencies
		TRAYNUMBER=1		# 1 selects tray 2, sigh...
		;;
	    *)
		echo "ERROR: printer $PRINTER does not support printing on transparencies, or else this script is out of date!"
		exit 1
		;;
	    esac
	    ;;
	*twoside)			# duplex printing
	    DUPLEX=true
	    case $PRINTER in		# supply initial empty page on certain printers
	    jwb128|jwb229-2|jwb320-2|inscc300|lex120|lex3fl|jwb120)	# these already start job on new page
		true
		;;
	    *)
		# echo "statusdict begin firstside { showpage } if end" >>$LPRFILE
		# LaserJet 5si broke the above code; a simple showpage works
		# on both 4si and 5si, sigh...
		echo "showpage" >>$LPRFILE
		;;
	    esac

	    ;;
	esac

	# Select duplex mode
	echo "statusdict begin $DUPLEX setduplexmode end"  >>$LPRFILE

	case "${RESOLUTION}dpi" in
	[0-9][0-9]*dpi)
	    # Output resolution setting, but executable only if PostScript is Level 2 or later
	    echo "/languagelevel where {pop languagelevel} {1} ifelse" >>$LPRFILE
	    echo "2 lt {} {<< /HWResolution [ $RESOLUTION $RESOLUTION ] >> setpagedevice} ifelse" >>$LPRFILE
	    ;;
	dpi)
	    ;;
	*)
	    echo "WARNING: Illegal RESOLUTION value [" $RESOLUTION "] ignored"
	    exitcode=1
	    ;;
	esac

	# [11-Jan-2001] NEW!
	# Select transparency tray
	if test "$TRAYSELECTION" = "true"
	then
	    echo "statusdict begin $TRAYNUMBER setpapertray end" >>$LPRFILE
	fi

	# Append the original PostScript file
	cat $file >>$LPRFILE

	# Replace hyphen by underscore in file names to prevent truncation by stupid lpr...
	jobname="`echo $userfile | sed -e 's/-/_/g'`"

	lpr -#$COPIES -J"$jobname" -T"$jobname" $LPRFILE

	/bin/rm -f $LPRFILE $OUTFILE
	;;

    *)
	echo "WARNING: File $userfile not printed: not PostScript, or DVI,  GIF, JPEG, PNG, Sun raster, Tektronix, TIFF, or X79"
	exitcode=1
	;;
    esac
done


echo PRINTER=$PRINTER
lpq -P $PRINTER
exit $exitcode
